/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : rtps.c
* Version      : 1.00
* Description  : This file is rBLE Application.
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version Description
*         : DD.MM.2019 1.00     First Release
***********************************************************************************************************************/

/***********************************************************************************************************************
* INCLUDE FILES
***********************************************************************************************************************/

#include    "rwke_api.h"

#include    "prf_config.h"
#include    "prf_sel.h"

#if (PRF_SEL_RTPS)
#define DATA_EVENT_MAX 3

typedef enum RTPS_STATUS_enum
{
    RTPS_STATUS_IDLE,
    RTPS_STATUS_CONNECTED
} RTPS_STASUS;

typedef struct RTPS_INFO
{
    RTPS_STASUS             status;
    RBLE_RTPS_EVENT_HANDLER callback;
    uint16_t                conhdl;
    uint16_t                notify_en;
    uint16_t                set_data_hdl[DATA_EVENT_MAX];
    uint16_t                wp;
    uint16_t                rp;
    RBLE_GATT_EVENT         event_data;
} RTPS_INFO_t;

RTPS_INFO_t rtps_info = {
    RTPS_STATUS_IDLE,       /* status */
    NULL,                   /* callback */
    0,                      /* conhdl */
    RBLE_RRTP_STOP_NTFIND,  /* notify en */
    {0},                    /* set_data_hdl */
    0,                      /* wp */
    0,                      /* rp */
    {0}                     /* event_data */
};

static void        rtps_send_enable_complete(RBLE_STATUS status);
static void        rtps_send_chg_IndNtf(uint8_t char_code, uint8_t * value);
static RBLE_STATUS rtps_set_data(uint16_t val_hdl, uint8_t * data, uint16_t data_len);
static void        rtps_send_response(uint16_t att_hdl, RBLE_STATUS result);
static void        gatt_callback(RBLE_GATT_EVENT * event);

/***********************************************************************************************************************
* FUNCTION DEFINITIONS
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: RBLE_RRTP_Server_Enable
* Description  : This task is performed server enabling.
*
* Arguments    : (st_init_api_info_t *) init_api_info_t -
*                    The location of the pointer to put the device configuration data
*                (uint16_t) conhdl -
*                    The connection handler
*                (uint8_t) con_type -
*                    The type of connection
*                (RBLE_RRTP_SERVER_PARAM *) param -
*                    The pointer of parameter
*                (RBLE_RTPS_EVENT_HANDLER) call_back -
*                    The pointer of call back function
* Return Value : (RBLE_STATUS) result -
*                  - RBLE_OK  :00h - normal end
*                  - RBLE_PARAM_ERR :F3h - parameter error
*                  - RBLE_STATUS_ERROR :F2h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_RRTP_Server_Enable(uint16_t conhdl,
                                    uint8_t  con_type,
                                    RBLE_RRTP_SERVER_PARAM * param,
                                    RBLE_RTPS_EVENT_HANDLER call_back)
{
    RBLE_STATUS result = RBLE_OK;
    
    if (RTPS_STATUS_IDLE == rtps_info.status)
    {
        /* parameter check */
        if ((param != NULL) && (call_back != NULL))
        {
            /* initialise GATT */
            result = RBLE_GATT_Enable(gatt_callback);
            if (RBLE_OK == result)
            {
                /* initialise rtps info */
                memset(&rtps_info,0,sizeof(rtps_info));
                rtps_info.callback = call_back;
                rtps_info.conhdl   = conhdl;

                if (RBLE_RRTP_CON_NORMAL == con_type)
                {
                    rtps_info.notify_en = param->data_ntf_en;
                }
                else
                {
                    rtps_info.notify_en = RBLE_RRTP_STOP_NTFIND;
                }
                /* set notify client configuration */
                result = rtps_set_data(RRTS_HDL_NOTIFY_CFG, (uint8_t*)&rtps_info.notify_en, 2);
            }
        }
        else
        {
            /* parameter error */
            result = RBLE_PARAM_ERR;
        }
    }
    else
    {
        /* status error */
        result = RBLE_STATUS_ERROR;
    }
    
    return result;
}

/***********************************************************************************************************************
* Function Name: RBLE_RRTP_Server_Disable
* Description  : This task is performed server disabling.
*
* Arguments    : (uint16_t) conhdl -
*                    The connection handler
* Return Value : (RBLE_STATUS) result -
*                  - RBLE_OK  :00h - normal end
*                  - RBLE_PARAM_ERR :F3h - parameter error
*                  - RBLE_STATUS_ERROR :F2h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_RRTP_Server_Disable(uint16_t conhdl)
{
    RBLE_STATUS     result = RBLE_OK;
    RBLE_RTPS_EVENT rrtp_event;
    
    if (RTPS_STATUS_CONNECTED == rtps_info.status)
    {
        if (conhdl == rtps_info.conhdl)
        {
            rtps_info.status                       = RTPS_STATUS_IDLE;

            /* disable complete callback */
            rrtp_event.type                        = RBLE_RRTP_EVENT_SERVER_DISABLE_COMP;
            rrtp_event.param.server_disable.conhdl = conhdl;
            rrtp_event.param.server_disable.status = RBLE_OK;
            
            rtps_info.callback(&rrtp_event);
        }
        else
        {
            /* parameter error */
            result = RBLE_PARAM_ERR;
        }
    }
    else
    {
        /* status error */
        result = RBLE_STATUS_ERROR;
    }
    
    return result;
}

/***********************************************************************************************************************
* Function Name: RBLE_RRTP_Server_Data_Transfer
* Description  : This task is transfer the packet data.
*
* Arguments    : (uint16_t) conhdl -
*                    The connection handler
*                (RBLE_RRTP_TRANSFER_DATA *) notify_info -
*                    The pointer of structure of the data packet
*                (uint16_t) Notify_No -
*                    The order of More_Data in connection interval
* Return Value : (RBLE_STATUS) result -
*                  - RBLE_OK  :00h - normal end
*                  - RBLE_PARAM_ERR :F3h - parameter error
*                  - RBLE_STATUS_ERROR :F2h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_RRTP_Server_Data_Transfer(uint16_t conhdl,
                                         RBLE_RRTP_TRANSFER_DATA * notify_info, uint16_t Notify_No)
{
    static const uint16_t val_hdl_map[] = { RRTS_HDL_NOTIFY_VAL1,
                                            RRTS_HDL_NOTIFY_VAL2,
                                            RRTS_HDL_NOTIFY_VAL3,
                                            RRTS_HDL_NOTIFY_VAL4 };

    RBLE_STATUS result;
    
    if (RTPS_STATUS_CONNECTED == rtps_info.status)
    {
        if ((conhdl == rtps_info.conhdl) &&
            (rtps_info.notify_en & RBLE_RRTP_START_NTF))
        {
            /* set data */
            result = rtps_set_data(val_hdl_map[Notify_No], notify_info->data, notify_info->data_len);
        }
        else
        {
            /* parameter error */
            result = RBLE_PARAM_ERR;
        }
    }
    else
    {
        /* status error */
        result = RBLE_STATUS_ERROR;
    }
    
    return result;
}

/***********************************************************************************************************************
* Function Name: rtps_send_enable_complete
* Description  : This task is 
*
* Arguments    : (RBLE_STATUS) status -
*                    The connection handler
* Return Value : None
***********************************************************************************************************************/
static void rtps_send_enable_complete(RBLE_STATUS status)
{
    RBLE_RTPS_EVENT rrtp_event;
    
    if (RBLE_OK == status)
    {
        rtps_info.status = RTPS_STATUS_CONNECTED;
    }

    /* enable complete callback */
    rrtp_event.type                       = RBLE_RRTP_EVENT_SERVER_ENABLE_COMP;
    rrtp_event.param.server_enable.conhdl = rtps_info.conhdl;
    rrtp_event.param.server_enable.status = status;
    rtps_info.callback(&rrtp_event);
}

/***********************************************************************************************************************
* Function Name: rtps_send_chg_IndNtf
* Description  : This task is send the Indication/Notification change
*
* Arguments    : (uint8_t) char_code -
*                    The internal code for characteristic
*                (uint8_t *) value -
* Return Value : None
***********************************************************************************************************************/
static void rtps_send_chg_IndNtf(uint8_t char_code, uint8_t * value)
{
    RBLE_RTPS_EVENT rrtp_event;
    uint16_t tmp_data;
    
    memcpy(&tmp_data, value, 2);

    if (RBLE_RRTP_RRTS_NTF_CFG == char_code)
    {
        rtps_info.notify_en = tmp_data;
    }
    else
    {
        /* Do nothing */
        __nop();
    }

    /* send write callback */
    rrtp_event.type                       = RBLE_RRTP_EVENT_SERVER_CHG_INDNTF_IND;
    rrtp_event.param.cfg_indntf.conhdl    = rtps_info.conhdl;
    rrtp_event.param.cfg_indntf.char_code = char_code;
    rrtp_event.param.cfg_indntf.cfg_val   = tmp_data;
    rtps_info.callback(&rrtp_event);
}

/***********************************************************************************************************************
* Function Name: rtps_set_data
* Description  : This task is set the data for transfer
*
* Arguments    : (uint16_t) val_hdl -
*                    The value handler
*                (uint8_t *) data -
*                    The pointer of the data
*                (uint16_t) data_len -
*                    The data length
* Return Value : (RBLE_STATUS) result -
***********************************************************************************************************************/
static RBLE_STATUS rtps_set_data(uint16_t val_hdl, uint8_t * data, uint16_t data_len)
{
    RBLE_STATUS          result;
    RBLE_GATT_NOTIFY_REQ notify;
    RBLE_GATT_SET_DATA   set_data;
    void               * Wr_p;

    switch (val_hdl)
    {
        case RRTS_HDL_NOTIFY_VAL1:
        case RRTS_HDL_NOTIFY_VAL2:
        case RRTS_HDL_NOTIFY_VAL3:
        case RRTS_HDL_NOTIFY_VAL4:
            {
                if (DB_TYPE_128BIT_UUID == atts_desc_list[val_hdl].type)
                {
                    Wr_p = ((struct atts_elmt_128 *)(atts_desc_list[val_hdl].value))->value;
                }
                else
                {
                    Wr_p = atts_desc_list[val_hdl].value;
                }
                memcpy(Wr_p, data, data_len);
                atts_desc_list[val_hdl].length = (uint8_t)data_len;

                /* send notify */
                notify.conhdl  = rtps_info.conhdl;
                notify.charhdl = val_hdl;
                result         = RBLE_GATT_Notify_Request(&notify);
            }
            break;
        default:
            {
                set_data.val_len = data_len;
                set_data.val_hdl = val_hdl;
                memcpy(&set_data.value[0], data, data_len);

                rtps_info.set_data_hdl[rtps_info.wp] = val_hdl;
                rtps_info.wp                         = (rtps_info.wp + 1) % DATA_EVENT_MAX;

                result                               = RBLE_GATT_Set_Data(&set_data);
            }
            break;
    } 
    return result;
}

/***********************************************************************************************************************
* Function Name: rtps_send_response
* Description  : This task is send response
*
* Arguments    : (uint16_t) att_hdl -
*                    The attribute handler
*                (RBLE_STATUS) result -
*                    The result data
* Return Value : None
***********************************************************************************************************************/
static void rtps_send_response(uint16_t att_hdl, RBLE_STATUS result)
{
    RBLE_GATT_WRITE_RESP wr_resp;
    
    if (result != RBLE_OK)
    {
        wr_resp.att_code = RBLE_ATT_ERR_APP_ERROR;
    }
    else
    {
        wr_resp.att_code = RBLE_ATT_ERR_NO_ERROR;
    }

    wr_resp.conhdl  = rtps_info.conhdl;
    wr_resp.att_hdl = att_hdl;

    RBLE_GATT_Write_Response(&wr_resp);
    
    return;
}

/***********************************************************************************************************************
* Function Name: gatt_callback
* Description  : This task is entry of GATT callback
*
* Arguments    : (RBLE_GATT_EVENTi *) event -
*                    The pointer of structure of event
* Return Value : None
***********************************************************************************************************************/
static void gatt_callback(RBLE_GATT_EVENT * event)
{
    RBLE_STATUS     result = RBLE_OK;
    uint16_t        tmp_rp, val_len;
    
    switch (event->type)
    {
        case RBLE_GATT_EVENT_WRITE_CMD_IND:
            {
                val_len = 0;
                switch (event->param.write_cmd_ind.elmt)
                {
                    case RRTS_HDL_NOTIFY_CFG:
                        {
                            val_len = 2;
                        }
                        break;
                    default:
                        {
                            /* could not come here */
                            __nop();
                        }
                        break;
                }

                if (val_len != 0)
                {
                    /* update data base */
                    rtps_info.event_data = *event;
                    result               = rtps_set_data(event->param.write_cmd_ind.elmt,
                                                         &event->param.write_cmd_ind.value[0], val_len);
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }

                if ((result != RBLE_OK) || (0 == val_len))
                {
                    if (TRUE == event->param.write_cmd_ind.resp)
                    {
                        /* send error response */
                        rtps_send_response(event->param.write_cmd_ind.elmt, RBLE_ERR);
                    }
                    else
                    {
                        /* Do nothing */
                        __nop();
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        case RBLE_GATT_EVENT_SET_DATA_CMP:
            {
                tmp_rp = rtps_info.rp;

                /* update read point */
                rtps_info.rp = (rtps_info.rp + 1) % DATA_EVENT_MAX;

                result = event->param.set_data_cmp.status;

                switch (rtps_info.set_data_hdl[tmp_rp])
                {
                    case RRTS_HDL_NOTIFY_CFG:
                        {
                            if (RTPS_STATUS_CONNECTED == rtps_info.status)
                            {
                                if (RBLE_OK == result)
                                {
                                    /* send write event to app */
                                    rtps_send_chg_IndNtf(RBLE_RRTP_RRTS_NTF_CFG,
                                                         &rtps_info.event_data.param.write_cmd_ind.value[0]);
                                }
                                else
                                {
                                    /* Do nothing */
                                    __nop();
                                }

                                /* send error response */
                                if (TRUE == rtps_info.event_data.param.write_cmd_ind.resp)
                                {
                                    rtps_send_response(rtps_info.set_data_hdl[tmp_rp], result);
                                }
                                else
                                {
                                    /* Do nothing */
                                    __nop();
                                }
                            }
                            else
                            {
                                /* enable complete callback */
                                rtps_send_enable_complete(result);
                            }
                        }
                        break;
                    default:
                        {
                            __nop();
                        }
                        break;
                }
            }
            break;
        case RBLE_GATT_EVENT_NOTIFY_COMP:
        case RBLE_GATT_EVENT_RESP_TIMEOUT:
        case RBLE_GATT_EVENT_COMMAND_DISALLOWED_IND:
        default:
            {
                /* Do nothing */
                __nop();
            }
            break;
    }
}

#endif /* (PRF_SEL_RTPS) #ifdef USE_SAMPLE_PROFILE */

