/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2016 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : rble_app_common.c
* Version      : 1.00
* Description  : This file is rBLE Application.
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version Description
*         : DD.MM.2016 1.00     First Release
***********************************************************************************************************************/
#include "arch.h"
#include "rble_api.h"
#include "rble_external.h"
#include "rble_app_common.h"
#include "rble_app.h"

#include "prf_sel.h"

#if (PRF_SEL_RTPS) /* __ENABLE_USE_RRTP__ */
#include "rble_app_rrtp.h"
#endif /* (PRF_SEL_RTPS) __ENABLE_USE_RRTP__ */


/************************************************* Define Declaration *************************************************/

/************************************************* PRIVATE PROTOTYPE **************************************************/
static void RBLE_APP_CallBack(RBLE_MODE mode);

/* --------------------------------------------- Generic Access Profile --------------------------------------------- */
static void        RBLE_APP_GAP_CallBack(RBLE_GAP_EVENT * event);       /* GAP Call Back */
static RBLE_STATUS GAP_Reset(void);                                     /* A GAP_Reset command is executed. */
static BOOL        GAP_Set_Name(void);                                  /* A GAP_Set_Name command is executed. */
static RBLE_STATUS GAP_Broadcast_Enable(void);                          /* A GAP_Broadcast_Enable command is executed. */
static RBLE_STATUS GAP_Broadcast_Disable(void);                         /* A GAP_Broadcast_Disable command is executed. */
static RBLE_STATUS GAP_Set_Bonding_Mode(void);                          /* A GAP_Set_Bonding_Mode command is executed. */
static RBLE_STATUS GAP_Disconnect(void);                                /* A GAP_Disconnect command is executed. */
static RBLE_STATUS GAP_Bonding_Response(void);                          /* A GAP_Bonding_Response command is executed. */

/* ------------------------------------------------ Security Manager ------------------------------------------------ */
static void        RBLE_APP_SM_CallBack(RBLE_SM_EVENT * event);         /* SM Call Back */
static RBLE_STATUS SM_Tk_Req_Resp(void);                                /* A SM_Tk_Req_Resp command is executed. */
static RBLE_STATUS SM_Ltk_Req_Resp(void);                               /* A SM_Ltk_Req_Resp command is executed. */

/* ------------------------------------------------ Vendor Specific ------------------------------------------------- */
static void        RBLE_APP_VS_CallBack(RBLE_VS_EVENT * event);         /* Vendor Specific Call Back */
static RBLE_STATUS VS_Enable(void);                                     /* Vendor Specific Command Enable */
static RBLE_STATUS VS_RF_Control(uint8_t rf_ctrl_cmd);                  /* RF Control */

/* ---------------------------------------------------- Utility ----------------------------------------------------- */
int_t RBLE_Get_Random(void);

/********************************************* PUBLIC Memory Declaration **********************************************/


/********************************************* PRIVATE Memory Declaration *********************************************/
RBLE_BD_ADDR                     Remote_Device                = { 0x5F, 0xB6, 0x05, 0xDC, 0x1B, 0x00 };
uint16_t                         Connection_Handle            = ILLEGAL_CONNECTION_HANDLE;
uint8_t                          Adv_Flag                     = FALSE;
uint8_t                          Connection_Idx               = 0xff;
uint8_t                          Connection_Type              = 0xff;
RBLE_APP_KEY_DATA                RBLE_APP_Key;
uint8_t                          Peer_Addr_Type               = RBLE_ADDR_PUBLIC;
BOOL                             Bonding_Flg                  = FALSE;
BOOL                             Measurement_Send_Flag        = FALSE;
BOOL                             MeasurementContext_Send_Flag = FALSE;
BOOL                             Ra_Control_Point_Flag        = FALSE;

uint32_t                         Remote_Sign_Counter;
BOOL                             InitFlag                     = FALSE;
BOOL                             Rf_Flag                      = FALSE;
BOOL                             TimeSynchronized_Flag        = FALSE;

/* RBLE APP STATUS */
RBLE_APP_STATUS                  rbleAppStatus                = RBLE_APP_STATUS_NONE;

uint16_t                         g_rble_rf_status;

/* device name */
static uint8_t                   device_name[RF_DEVICE_NAME_SZ];

/* I/O capabilities */
static enum RBLE_GAP_IO_CAP_enum iocap;

/* Authentication requirement */
static enum RBLE_AUTH_REQ_enum   auth_req;

/* Advertise flag */
static bool                      startAdvertiseFlag;

/***********************************************************************************************************************
* Function Name: RBLE_App_InitializeAPIRequested
* Description  : This task is performed by the rBLE_APP initial setting.
*
* Arguments    : (st_init_api_info_t *) init_api_info_t -
*                    The location of the pointer to put the device configuration data
* Return Value : (BOOL) Ret -
*                  - TRUE  :01h - normal end
*                  - FALSE :00h - status error
***********************************************************************************************************************/
bool RBLE_App_InitializeAPIRequested(st_init_api_info_t * init_api_info_t)
{
    BOOL        Ret = FALSE;
    RBLE_STATUS Ret_Status;

    if (RBLE_APP_STATUS_NONE == rbleAppStatus)
    {
        /* parameter check */
        if ((NULL == init_api_info_t->device_name) ||
           ((init_api_info_t->auth != RBLE_AUTH_REQ_NO_MITM_BOND) && (init_api_info_t->auth != RBLE_AUTH_REQ_NO_MITM_NO_BOND)))
        {
            rbleAppStatus = RBLE_APP_STATUS_NONE;
        }
        else
        {
            /* Save the device name to RAM */
            memcpy(device_name, init_api_info_t->device_name, sizeof(device_name));
            device_name[RBLE_BD_NAME_SIZE - 1] = '\0';

            /* Save the I/O capabilities to RAM */
            /* I/O capabilities is "Input: No Output: No" fix */
            iocap = RBLE_IO_CAP_NO_INPUT_NO_OUTPUT;

            /* Save the Authentication requirement to RAM */
            auth_req = init_api_info_t->auth;

            /* clear static variable */
            Adv_Flag                     = FALSE;
            Connection_Idx               = 0xff;
            Connection_Type              = 0xff;
            Bonding_Flg                  = FALSE;
            Measurement_Send_Flag        = FALSE;
            MeasurementContext_Send_Flag = FALSE;
            Ra_Control_Point_Flag        = FALSE;
            Rf_Flag                      = FALSE;
            InitFlag                     = FALSE;

            /* GAP reset */
            Ret_Status = GAP_Reset();
            if (RBLE_OK != Ret_Status)
            {
                Ret = FALSE;
                rbleAppStatus = RBLE_APP_STATUS_NONE;
            }
            else
            {
                /* State update */
                rbleAppStatus = RBLE_APP_STATUS_INIT_AP_REQ;
                InitFlag = TRUE;
            }
        }
    }
    else
    {
        rbleAppStatus = RBLE_APP_STATUS_NONE;
    }

    return(Ret);
}


/***********************************************************************************************************************
* Function Name: RBLE_App_EnableRfRequested
* Description  : This task is performed by the rBLE_APP wireless communication enabled.
*
* Arguments    : (bool) startAdvertise -
*                    Advetise propriety
* Return Value : (BOOL) Ret -
*                  - TRUE  :01h - normal end
*                  - FALSE :00h - status error
***********************************************************************************************************************/
bool RBLE_App_EnableRfRequested(bool startAdvertise)
{
    BOOL Ret = FALSE;

    if ((RBLE_APP_STATUS_NONE == rbleAppStatus) &&
       (TRUE == InitFlag) &&
       (FALSE == Rf_Flag))
    {
        /* Save the Advertise Flag */
        startAdvertiseFlag = startAdvertise;

        /* To control the power of the RF chip */
        if (RBLE_OK == GAP_Reset())
        {
            /* State update */
            rbleAppStatus = RBLE_APP_STATUS_RF_EFFECTIVE_REQ;
            Ret           = TRUE;
        }
        else
        {
            rbleAppStatus = RBLE_APP_STATUS_NONE;
        }
    }
    else
    {
        rbleAppStatus = RBLE_APP_STATUS_NONE;
    }

    return(Ret);
}


/***********************************************************************************************************************
* Function Name: RBLE_App_DisableRfRequested
* Description  : This task is performed by the rBLE_APP wireless communication disabled.
*
* Arguments    : None
* Return Value : (BOOL) Ret -
*                  - TRUE  :01h - normal end
*                  - FALSE :00h - status error
***********************************************************************************************************************/
bool RBLE_App_DisableRfRequested(void)
{
    BOOL Ret = FALSE;

    if ((RBLE_APP_STATUS_NONE == rbleAppStatus) &&
       (TRUE == InitFlag) &&
       (TRUE == Rf_Flag))
    {
        /* Wireless function disabled */
        if (RBLE_OK == VS_RF_Control(RBLE_VS_RFCNTL_CMD_POWDOWN))
        {
            /* State update */
            rbleAppStatus = RBLE_APP_STATUS_RF_DISABLE_REQ;
            Ret = TRUE;
        }
        else
        {
            rbleAppStatus = RBLE_APP_STATUS_NONE;
        }
    }
    else
    {
        rbleAppStatus = RBLE_APP_STATUS_NONE;
    }

    return(Ret);
}


/***********************************************************************************************************************
* Function Name: RBLE_App_StartAdvertiseRequested
* Description  : This task, to start the rBLE_APP advertise.
*
* Arguments    : None
* Return Value : (BOOL) Ret -
*                  - TRUE  :01h - normal end
*                  - FALSE :00h - status error
***********************************************************************************************************************/
bool RBLE_App_StartAdvertiseRequested(void)
{
    BOOL Ret = FALSE;

    if ((RBLE_APP_STATUS_NONE == rbleAppStatus) &&
       (TRUE == InitFlag) &&
       (TRUE == Rf_Flag) &&
       (FALSE == Adv_Flag) &&
       (ILLEGAL_CONNECTION_HANDLE == Connection_Handle))
    {
        /* Wireless function disabled */
        if (RBLE_OK == GAP_Broadcast_Enable())
        {
            /* State update */
            rbleAppStatus = RBLE_APP_STATUS_ADVERTISE_START;
            Ret = TRUE;
        }
        else
        {
            rbleAppStatus = RBLE_APP_STATUS_NONE;
        }
    }
    else
    {
        rbleAppStatus = RBLE_APP_STATUS_NONE;
    }

    return(Ret);
}


/***********************************************************************************************************************
* Function Name: RBLE_App_StopAdvertiseRequested
* Description  : This task, to stop the rBLE_APP advertise.
*
* Arguments    : None
* Return Value : (BOOL) Ret -
*                  - TRUE  :01h - normal end
*                  - FALSE :00h - status error
***********************************************************************************************************************/
bool RBLE_App_StopAdvertiseRequested(void)
{
    BOOL Ret = FALSE;

    if ((RBLE_APP_STATUS_NONE == rbleAppStatus) &&
       (TRUE == InitFlag) &&
       (TRUE == Rf_Flag) &&
       (TRUE == Adv_Flag) &&
       (ILLEGAL_CONNECTION_HANDLE == Connection_Handle))
    {
        /* Wireless function disabled */
        if (RBLE_OK == GAP_Broadcast_Disable())
        {
            /* State update */
            rbleAppStatus = RBLE_APP_STATUS_ADVERTISE_STOP;
            Ret = TRUE;
        }
        else
        {
            rbleAppStatus = RBLE_APP_STATUS_NONE;
        }
    }
    else
    {
        rbleAppStatus = RBLE_APP_STATUS_NONE;
    }

    return(Ret);
}

/***********************************************************************************************************************
* Function Name: RBLE_App_DisconnectedRequested
* Description  : This task is performed by the rBLE_APP disconnection.
*
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
void RBLE_App_DisconnectedRequested(void)
{
    if ((RBLE_APP_STATUS_NONE == rbleAppStatus) &&
       (TRUE == InitFlag) &&
       (TRUE == Rf_Flag) &&
       (Connection_Handle != ILLEGAL_CONNECTION_HANDLE))
    {
        /* Disconnected */
        if (RBLE_OK == GAP_Disconnect())
        {
            /* State update */
            rbleAppStatus = RBLE_APP_STATUS_DISCONNECT;
        }
        else
        {
            rbleAppStatus = RBLE_APP_STATUS_NONE;
        }
    }
    else
    {
        rbleAppStatus = RBLE_APP_STATUS_NONE;
    }

    return;
}

/***********************************************************************************************************************
* Function Name: RBLE_App_CancelRequested
* Description  : This task is performed by the rBLE_APP command cancel.
*
* Arguments    : None
* Return Value : (BOOL) Ret -
*                  - TRUE  :01h - normal end
*                  - FALSE :00h - status error
***********************************************************************************************************************/
bool RBLE_App_CancelRequested(void)
{
    return(TRUE);
}

/***********************************************************************************************************************
* Function Name: RBLE_App_Init
* Description  : This task is performed the rBLE_APP initialization callback.
*
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
bool RBLE_App_Init(void)
{
    /* Initialize rBLE */
    RBLE_Init(&RBLE_APP_CallBack);

    return(TRUE);
}


/******************** Call Back API ********************/

/***********************************************************************************************************************
* Function Name: RBLE_APP_CallBack
* Description  : This task is performed the rBLE_APP callback.
*
* Arguments    : (RBLE_MODE) mode -
*                    To hold the RBLE mode
* Return Value : None
***********************************************************************************************************************/
static void RBLE_APP_CallBack(RBLE_MODE mode)
{
    switch (mode)
    {
        case RBLE_MODE_INITIALIZE:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_MODE_ACTIVE:
            {
                /* To enable the VS function */
                VS_Enable();

                /* Wireless function disabled */
                VS_RF_Control(RBLE_VS_RFCNTL_CMD_POWDOWN);
            }
            break;
        case RBLE_MODE_RESET:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_MODE_SLEEP:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_MODE_ERROR:
            {
                /* Do nothing */
                __nop();
            }
            break;
        default:
            {
                /* Do nothing */
                __nop();
            }
            break;
    }
}

/***********************************************************************************************************************
* Function Name: RBLE_APP_GAP_CallBack
* Description  : This task is performed the rBLE_APP_GAP Call Back.
*
* Arguments    : (RBLE_GAP_EVENT *) event -
*                    A pointer to the structure that stores the value of event
* Return Value : None
***********************************************************************************************************************/
static void RBLE_APP_GAP_CallBack(RBLE_GAP_EVENT * event)
{
    RBLE_CONNECT_INFO * Con_Info_p;

    switch (event->type)
    {
        /* Reset completion event */
        case RBLE_GAP_EVENT_RESET_RESULT:
            {
                /* If status is init request or rf effective request */
                if ((RBLE_APP_STATUS_INIT_AP_REQ == rbleAppStatus) || (RBLE_APP_STATUS_RF_EFFECTIVE_REQ == rbleAppStatus))
                {
                    if (RBLE_OK == event->param.status)
                    {
                        /* set device Name */
                        if (RBLE_OK != GAP_Set_Name())
                        {
                            rbleAppStatus = RBLE_APP_STATUS_NONE;
                        }
                        else
                        {
                            /* Do nothing */
                            __nop();
                        }
                    }
                    else
                    {
                        /* Do nothing */
                        __nop();
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        /* Device name setup completion event */
        case RBLE_GAP_EVENT_SET_NAME_COMP:
            {
                /* If status is initialized */
                if (RBLE_APP_STATUS_INIT_AP_REQ == rbleAppStatus)
                {
                    if (RBLE_OK == event->param.status)
                    {
                        /* Enables VS. */
                        if (RBLE_OK != VS_Enable())
                        {
                            rbleAppStatus = RBLE_APP_STATUS_NONE;
                        }
                        else
                        {
                            /* Controls the power supply of the RF chip. */
                            if (RBLE_OK != VS_RF_Control(RBLE_VS_RFCNTL_CMD_POWDOWN))
                            {
                                rbleAppStatus = RBLE_APP_STATUS_NONE;
                            }
                            else
                            {
                                /* Do nothing */
                                __nop();
                            }
                        }
                    }
                    else
                    {
                        /* Do nothing */
                        __nop();
                    }
                }
                /* If the status is rf effective request */
                else if (RBLE_APP_STATUS_RF_EFFECTIVE_REQ == rbleAppStatus)
                {
                    if (RBLE_OK == event->param.status)
                    {
                        /* Enables VS. */
                        if (RBLE_OK != VS_Enable())
                        {
                            rbleAppStatus = RBLE_APP_STATUS_NONE;
                        }
                        else
                        {
                            /* If the processing of the advertisement is required */
                            if (TRUE == startAdvertiseFlag)
                            {
                                /* Enables broadcasting */
                                if (RBLE_OK != GAP_Broadcast_Enable())
                                {
                                    rbleAppStatus = RBLE_APP_STATUS_NONE;
                                }
                                else
                                {
                                    /* Do nothing */
                                    __nop();
                                }
                            }
                            else
                            {
                                rbleAppStatus = RBLE_APP_STATUS_NONE;
                                Rf_Flag       = TRUE;

                                g_rble_rf_status = RBLE_RF_ENABLED;
                            }
                        }
                    }
                    else
                    {
                        rbleAppStatus = RBLE_APP_STATUS_NONE;
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        /* Broadcast enable event */
        case RBLE_GAP_EVENT_BROADCAST_ENABLE_COMP:
            {
                /* If status is rf effective request or advertise start */
                if ((RBLE_APP_STATUS_RF_EFFECTIVE_REQ == rbleAppStatus) || (RBLE_APP_STATUS_ADVERTISE_START == rbleAppStatus))
                {
                    if (RBLE_OK == event->param.status)
                    {
                        rbleAppStatus    = RBLE_APP_STATUS_NONE;
                        Adv_Flag         = TRUE;
                        Rf_Flag          = TRUE;

                        g_rble_rf_status = RBLE_RF_ADVERTISING;
                    }
                    else
                    {
                        rbleAppStatus = RBLE_APP_STATUS_NONE;
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        /* Broadcast disable event */
        case RBLE_GAP_EVENT_BROADCAST_DISABLE_COMP:
            {
                /* If status is advertise stop */
                if (RBLE_APP_STATUS_ADVERTISE_STOP == rbleAppStatus)
                {
                    if (RBLE_OK == event->param.status)
                    {
                        rbleAppStatus = RBLE_APP_STATUS_NONE;
                        Adv_Flag      = FALSE;
                    }
                    else
                    {
                        /* Do nothing */
                        __nop();
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        /* Bonding mode setup event */
        case RBLE_GAP_EVENT_SET_BONDING_MODE_COMP:
            {
                /* If status is rf effective request */
                if (RBLE_APP_STATUS_RF_EFFECTIVE_REQ == rbleAppStatus)
                {
                    if (RBLE_OK == event->param.status)
                    {
                        /* start Advertise Flag is TRUE */
                        if (TRUE == startAdvertiseFlag)
                        {
                            /* Effective setting of broadcast */
                            if (RBLE_OK != GAP_Broadcast_Enable())
                            {
                                rbleAppStatus = RBLE_APP_STATUS_NONE;
                            }
                            else
                            {
                                /* Do nothing */
                                __nop();
                            }
                        }
                        else
                        {
                            rbleAppStatus = RBLE_APP_STATUS_NONE;
                            Rf_Flag       = TRUE;
                        }
                    }
                    else
                    {
                        /* Do nothing */
                        __nop();
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        /* Security mode setup event */
        case RBLE_GAP_EVENT_SET_SECURITY_REQUEST_COMP:
            {
                /* If status is rf effective request */
                if (RBLE_APP_STATUS_RF_EFFECTIVE_REQ == rbleAppStatus)
                {
                    if (RBLE_OK == event->param.status)
                    {
                        /* Setting of the bonding mode */
                        if (RBLE_OK != GAP_Set_Bonding_Mode())
                        {
                            rbleAppStatus = RBLE_APP_STATUS_NONE;
                        }
                        else
                        {
                            /* Do nothing */
                            __nop();
                        }
                    }
                    else
                    {
                        rbleAppStatus = RBLE_APP_STATUS_NONE;
                    }
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        /* LE link connection event */
        case RBLE_GAP_EVENT_CONNECTION_COMP:
            {
                if (RBLE_OK == event->param.status)
                {
                    Con_Info_p        = &event->param.conn_comp.connect_info;
                    Connection_Handle = Con_Info_p->conhdl;
                    Adv_Flag          = FALSE;
                    Remote_Device     = Con_Info_p->peer_addr;
                    Peer_Addr_Type    = Con_Info_p->peer_addr_type;

                    rbleAppStatus     = RBLE_APP_STATUS_CONNECT;
                    g_rble_rf_status  = RBLE_RF_CONNECTED;

#if (PRF_SEL_RTPS) /*  __ENABLE_USE_RRTP__ */
                    /* To enable the Server */
                    if (RBLE_OK != RBLE_App_RrtpServerEnable())
                    {
                        rbleAppStatus = RBLE_APP_STATUS_NONE;
                    }
                    else
                    {
                        /* Do nothing */
                        __nop();
                    }
#endif /* (PRF_SEL_RTPS) __ENABLE_USE_RRTP__ */
                }
            }
            break;
        /* LE link disconnection completion event */
        case RBLE_GAP_EVENT_DISCONNECT_COMP:
            {
                rbleAppStatus                = RBLE_APP_STATUS_NONE;
                Connection_Handle            = ILLEGAL_CONNECTION_HANDLE;
                Measurement_Send_Flag        = FALSE;
                MeasurementContext_Send_Flag = FALSE;
                Ra_Control_Point_Flag        = FALSE;

                g_rble_rf_status             = RBLE_RF_ENABLED;
            }
            break;
        /* Bonding completion event */
        case RBLE_GAP_EVENT_BONDING_COMP:
            {
                Connection_Idx = event->param.bonding_comp.idx;
                Bonding_Flg    = FALSE;
            }
            break;
        /* Peer device bonding request notification event */
        case RBLE_GAP_EVENT_BONDING_REQ_IND:
            {
                Bonding_Flg = TRUE;
                /* Responds to a bonding request */
                if (RBLE_OK != GAP_Bonding_Response())
                {
                    rbleAppStatus = RBLE_APP_STATUS_NONE;
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        default:
            {
                /* Do nothing */
                __nop();
            }
            break;
    }
}


/***********************************************************************************************************************
* Function Name: GAP_Reset
* Description  : This task is performed the GAP_Reset.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS GAP_Reset(void)
{

    RBLE_STATUS Ret_Status;

    /* API Call */
    Ret_Status = RBLE_GAP_Reset(&RBLE_APP_GAP_CallBack, &RBLE_APP_SM_CallBack);

    Connection_Handle = ILLEGAL_CONNECTION_HANDLE;

    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: GAP_Set_Name
* Description  : This task is performed the GAP_set_name.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static BOOL GAP_Set_Name(void)
{
    static RBLE_BD_NAME bd_name_param;
    RBLE_STATUS         Ret_Status;

    /* length for name */
    bd_name_param.namelen = (uint8_t)(strlen((char *)device_name));

    /* array of bytes for name */
    memcpy(bd_name_param.name, device_name, RBLE_BD_NAME_SIZE);

    /* API Call */
    Ret_Status = RBLE_GAP_Set_Name(&bd_name_param);

    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: GAP_Broadcast_Enable
* Description  : This task is performed by the GAP_broadcast_enabled.

* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS GAP_Broadcast_Enable(void)
{
    RBLE_STATUS                     Ret_Status;
    static RBLE_ADV_INFO param =
    {
        {
            0x30,                                       /* AdvIntervalMin */
            0x60,                                       /* AdvIntervalMax */
            RBLE_GAP_ADV_CONN_UNDIR,                    /* AdvType        */
            RBLE_ADDR_PUBLIC,                           /* OwnAddrType    */
            RBLE_ADDR_PUBLIC,                           /* DirectAddrType */
            { 0xbc, 0x9a, 0x78, 0x56, 0x34, 0x12 },     /* DirectAddr     */
            RBLE_ADV_ALL_CHANNELS,                      /* AdvChannelMap  */
            RBLE_ADV_ALLOW_SCAN_ANY_CON_ANY,            /* AdvFiltPolicy  */
            0x00,                                       /* Reserved       */
        },
        {
            /* AdvDataLen     */
            3 + 3 + 4,
            /* AdvData        */
            2,  0x01, 0x06,                                                     /* Flags               */
            3,  0x03, 0x09, 0x18,
            2, 0x09, 'R', 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0, 0x0,    /* Complete local name */
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        },
        {
            /* ScanRspDataLen */
            0,
            /* ScanData       */
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00
        }
    };


    /* set device name */
    if (strlen((const char *)device_name) < 22)
    {
        memcpy(&param.adv_data.adv_data.data[9], device_name, strlen((const char *)device_name));
        param.adv_data.adv_data.data[7] = (strlen((const char *)device_name) + 1);
        param.adv_data.adv_data_len     = (strlen((const char *)device_name) + 2 + 3 + 4);
    }
    else
    {
        memcpy(&param.adv_data.adv_data.data[9], device_name, 22);
        param.adv_data.adv_data.data[7] = 23;
        param.adv_data.adv_data_len     = 31;
    }

    /* API Call */
    Ret_Status = RBLE_GAP_Broadcast_Enable(RBLE_GAP_GEN_DISCOVERABLE, RBLE_GAP_UND_CONNECTABLE, &param);

    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: GAP_Broadcast_Disable
* Description  : This task is performed by the GAP_broadcast_disabled.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS GAP_Broadcast_Disable(void)
{
    RBLE_STATUS Ret_Status;

    /* API Call */
    Ret_Status = RBLE_GAP_Broadcast_Disable();

    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: GAP_Set_Bonding_Mode
* Description  : This task is performed by the GAP_Set_Bonding_Mode.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS GAP_Set_Bonding_Mode(void)
{
#ifdef D_RBLE_SM_ENABLE
    const uint16_t mode = RBLE_GAP_BONDABLE;        /* GAP_BONDABLE */
#else
    const uint16_t mode = RBLE_GAP_NON_BONDABLE;    /* GAP_BONDABLE */
#endif

    RBLE_STATUS Ret_Status;

    /* API Call */
    Ret_Status = RBLE_GAP_Set_Bonding_Mode(mode);

    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: GAP_Disconnect
* Description  : This task is performed by the GAP_Disconnect.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS GAP_Disconnect(void)
{
    RBLE_STATUS Ret_Status;
    uint16_t con_hdl = Connection_Handle;

    /* Valid Connection Handle ? */
    if (ILLEGAL_CONNECTION_HANDLE == con_hdl)
    {
        /* Do nothing */
        __nop();
    }
    else
    {
        /* API Call */
        Ret_Status = RBLE_GAP_Disconnect(con_hdl);
    }
    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: GAP_Bonding_Response
* Description  : This task is performed by the GAP_Bonding_Response command.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS GAP_Bonding_Response(void)
{
    RBLE_STATUS             Ret_Status;
    static RBLE_BOND_RESP_PARAM Param = {
        0x0000,                                         /* Connection handle           */
        RBLE_OK,                                        /* accept or reject bonding    */
        RBLE_IO_CAP_NO_INPUT_NO_OUTPUT,                 /* IO capabilities             */
        RBLE_OOB_AUTH_DATA_NOT_PRESENT,                 /* OOB flag                    */
        RBLE_AUTH_REQ_NO_MITM_BOND,                     /* Authentication Requirements */
        RBLE_SMP_MAX_ENC_SIZE_LEN,                      /* Encryption key size         */
        RBLE_KEY_DIST_NONE,                             /* Initiator key distribution  */
#ifdef D_RBLE_SM_ENABLE
        RBLE_KEY_DIST_ENCKEY | RBLE_KEY_DIST_IDKEY,     /* Initiator key distribution  */
#else
        RBLE_KEY_DIST_NONE,                             /* Initiator key distribution  */
#endif
        0x00                                            /* Reserved                    */
    };

    /* set I/O capabilities */
    Param.io_cap = iocap;

    /* set Authentication Requirements */
    Param.auth_req = auth_req;

    /* Valid Connection Handle ? */
    if (ILLEGAL_CONNECTION_HANDLE != Connection_Handle)
    {
        Param.conhdl = Connection_Handle;

        /* API Call */
        Ret_Status   = RBLE_GAP_Bonding_Response(&Param);
    }
    else
    {
        /* Do nothing */
        __nop();
    }
    return(Ret_Status);
}

/* -------------------------------- Security Manager -------------------------------- */

/***********************************************************************************************************************
* Function Name: RBLE_APP_SM_CallBack
* Description  : This task is performed by the rBLE_SM Call Back.
*
* Arguments    : (RBLE_SM_EVENT *) event -
*                    A pointer to the structure that stores the value of event
* Return Value : None
***********************************************************************************************************************/
static void RBLE_APP_SM_CallBack(RBLE_SM_EVENT * event)
{
    switch (event->type)
    {
        case RBLE_SM_ENC_START_IND:         /* Encryption Start Indicate Event */
            {
                /* Do nothing */
                __nop();
            }
                break;
        case RBLE_SM_TK_REQ_IND:            /* TK Request Event */
            {
                Connection_Idx = event->param.tk_req.idx;
                SM_Tk_Req_Resp();
            }
                break;
        case RBLE_SM_LTK_REQ_IND:           /* LTK Request Event */
            {
                Connection_Idx = event->param.ltk_req.idx;
                SM_Ltk_Req_Resp();
            }
                break;
        case RBLE_SM_KEY_IND:               /* Key Indicate Event */
            {
                Connection_Idx = event->param.key_ind.idx;
                if (RBLE_SMP_KDIST_ENCKEY == event->param.key_ind.key_code)
                {
                    RBLE_APP_Key.EncKey         = event->param.key_ind;
                    RBLE_APP_Key.Key_Set_Flags |= (1 << RBLE_APP_ENCKEY);
                }
                else if (RBLE_SMP_KDIST_IDKEY == event->param.key_ind.key_code)
                {
                    RBLE_APP_Key.IrKey          = event->param.key_ind;
                    RBLE_APP_Key.Key_Set_Flags |= (1 << RBLE_APP_IDKEY);
                }
                else if (RBLE_SMP_KDIST_SIGNKEY == event->param.key_ind.key_code)
                {
                    RBLE_APP_Key.SignKey        = event->param.key_ind;
                    RBLE_APP_Key.Key_Set_Flags |= (1 << RBLE_APP_SIGNKEY);
                    Remote_Sign_Counter         = 0;
                }
                else
                {
                    /* Do nothing */
                    __nop();
                }
            }
            break;
        default:
            {
                /* Do nothing */
                __nop();
            }
            break;
    }
}

/***********************************************************************************************************************
* Function Name: SM_Tk_Req_Resp
* Description  : This task is performed by the SM_Tk_Req_Resp command.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS SM_Tk_Req_Resp(void)
{
    static RBLE_KEY_VALUE tk = {
            0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00      /* TkKey(for PassKey) */
    };
    RBLE_STATUS           Ret_Status;

    uint32_t              passkey;
    uint8_t             * byteptr = (uint8_t *)&passkey;
    uint8_t               i;

    /* Generate PassKey (range:000,000 - 999,999) */
    passkey  = (uint32_t)RBLE_Get_Random();
    passkey |= (uint32_t)RBLE_Get_Random() << 16;
    passkey %= 1000000;

    /* Copy PassKey to TK(Temporary Key) */
    for (i = 0; i < sizeof(uint32_t); i++)
    {
        tk.key[RBLE_KEY_LEN - 1 - i] = byteptr[i];
    }
    /* API Call */
    Ret_Status = RBLE_SM_Tk_Req_Resp(Connection_Idx, RBLE_OK, &tk);

    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: SM_Ltk_Req_Resp
* Description  : This task is performed by the SM_Ltk_Req_Resp command.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS SM_Ltk_Req_Resp(void)
{
    const uint8_t         sec_prop = 0x01;      /* SecProp */
    static RBLE_RAND_NB   nb       = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00      /* Rand */
    };
    static RBLE_KEY_VALUE ltk      = {
        0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00      /* LTK */
    };

    uint8_t               status   = RBLE_OK;
    uint16_t              ediv     = 0x0000;    /* Ediv */
    RBLE_STATUS           Ret_Status;
    int_t                 i;

    if (TRUE == Bonding_Flg)
    {
        if (0u == (RBLE_APP_Key.Key_Set_Flags & (1 <<  RBLE_APP_ENCKEY)))
        {

            for (i = 0; i <  RBLE_RAND_NB_LEN; i++)
            {
                RBLE_APP_Key.EncKey.nb.nb[i] = (uint8_t)RBLE_Get_Random();
            }

            for (i = 0; i <  RBLE_KEY_LEN; i++)
            {
                RBLE_APP_Key.EncKey.ltk.key[i] = (uint8_t)RBLE_Get_Random();
            }
            RBLE_APP_Key.EncKey.ediv    = 0x1234;
            RBLE_APP_Key.Key_Set_Flags |= (1 << RBLE_APP_ENCKEY);
        }
        else
        {
            /* Do nothing */
            __nop();
        }
        ediv = RBLE_APP_Key.EncKey.ediv;
        nb   = RBLE_APP_Key.EncKey.nb;
        ltk  = RBLE_APP_Key.EncKey.ltk;

    }
    else if (0u != (RBLE_APP_Key.Key_Set_Flags & (1 <<  RBLE_APP_ENCKEY)))
    {
        ediv = RBLE_APP_Key.EncKey.ediv;
        nb   = RBLE_APP_Key.EncKey.nb;
        ltk  = RBLE_APP_Key.EncKey.ltk;
    }
    else
    {
        status = RBLE_ERR;
    }

    /* API Call */
    Ret_Status = RBLE_SM_Ltk_Req_Resp(Connection_Idx, status, sec_prop, ediv, &nb, &ltk);

    return(Ret_Status);
}

/* ---------------------------- Vendor Specific ---------------------------- */

/***********************************************************************************************************************
* Function Name: RBLE_APP_VS_CallBack
* Description  : This task is performed by the Vendor_Specific Call Back.

* Arguments    : (RBLE_TIPC_EVENT *) event -
*                    A pointer to the structure that stores the value of event
* Return Value : None
***********************************************************************************************************************/
static void RBLE_APP_VS_CallBack(RBLE_VS_EVENT * event)
{
    switch (event->type)
    {
        /* RF power supply control completion event */
        case RBLE_VS_EVENT_RF_CONTROL_COMP:
            {
                /* If status is rf disable request */
                if (RBLE_APP_STATUS_RF_DISABLE_REQ == rbleAppStatus)
                {
                    rbleAppStatus = RBLE_APP_STATUS_NONE;
                    Rf_Flag       = FALSE;
                }
                /* If status is init request */
                else if (RBLE_APP_STATUS_INIT_AP_REQ == rbleAppStatus)
                {
                    if (RBLE_OK == event->param.status)
                    {
                        rbleAppStatus    = RBLE_APP_STATUS_NONE;
                        g_rble_rf_status = RBLE_RF_INITIALIZED;
                    }
                    else
                    {
                        rbleAppStatus = RBLE_APP_STATUS_NONE;
                    }
                }
            }
            break;
        /* other */
        default:
            {
                /* Do nothing */
                __nop();
            }
            break;
    }
}


/***********************************************************************************************************************
* Function Name: VS_Enable
* Description  : This task is performed by the Vendor_Specfic_Enable command.
*
* Arguments    : None
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS VS_Enable(void)
{
    RBLE_STATUS Ret_Status;

    /* API Call */
    Ret_Status = RBLE_VS_Enable(RBLE_APP_VS_CallBack);

    return(Ret_Status);
}


/***********************************************************************************************************************
* Function Name: VS_RF_Control
* Description  : This task is performed by the Vendor_Specfic_RF_Control command.
*
* Arguments    : (uint8_t) rf_ctrl_cmd
*                    Stores the type of command
* Return Value : (RBLE_STATUS) Ret_Status -
*                    - TRUE  :01h - Menu Continue
*                    - FALSE :00h - status error
***********************************************************************************************************************/
static RBLE_STATUS VS_RF_Control(uint8_t rf_ctrl_cmd)
{
    RBLE_STATUS Ret_Status;

    switch (rf_ctrl_cmd)
    {
        case RBLE_VS_RFCNTL_CMD_POWDOWN:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_VS_RFCNTL_CMD_POWUP_DDCON:
            {
                /* Do nothing */
                __nop();
            }
            break;
        case RBLE_VS_RFCNTL_CMD_POWUP_DDCOFF:
            {
                /* Do nothing */
                __nop();
            }
            break;
        default:
            {
                /* Do nothing */
                __nop();
            }
            break;
    }

    /* API Call */
    Ret_Status = RBLE_VS_RF_Control(rf_ctrl_cmd);

    return(Ret_Status);
}

/* ------------------------------------ Utility ------------------------------------- */

/***********************************************************************************************************************
* Function Name: RBLE_Get_Random
* Description  : This task is performed by the RBLE Get Random number.
*
* Arguments    : None
* Return Value : (int_t) rand
:                    random number
***********************************************************************************************************************/
int_t RBLE_Get_Random(void)
{
    return(rand());
}

