/***********************************************************************************************************************
* DISCLAIMER
* This software is supplied by Renesas Electronics Corporation and is only intended for use with Renesas products. No
* other uses are authorized. This software is owned by Renesas Electronics Corporation and is protected under all
* applicable laws, including copyright laws.
* THIS SOFTWARE IS PROVIDED "AS IS" AND RENESAS MAKES NO WARRANTIES REGARDING THIS SOFTWARE, WHETHER EXPRESS, IMPLIED
* OR STATUTORY, INCLUDING BUT NOT LIMITED TO WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
* NON-INFRINGEMENT. ALL SUCH WARRANTIES ARE EXPRESSLY DISCLAIMED.
* TO THE MAXIMUM EXTENT PERMITTED NOT PROHIBITED BY LAW, NEITHER RENESAS ELECTRONICS CORPORATION NOR ANY OF ITS
* AFFILIATED COMPANIES SHALL BE LIABLE FOR ANY DIRECT, INDIRECT, SPECIAL, INCIDENTAL OR CONSEQUENTIAL DAMAGES FOR ANY
* REASON RELATED TO THIS SOFTWARE, EVEN IF RENESAS OR ITS AFFILIATES HAVE BEEN ADVISED OF THE POSSIBILITY OF SUCH
* DAMAGES.
* Renesas reserves the right, without notice, to make changes to this software and to discontinue the availability of
* this software. By using this software, you agree to the additional terms and conditions found by accessing the
* following link:
* http://www.renesas.com/disclaimer
*
* Copyright (C) 2019 Renesas Electronics Corporation. All rights reserved.
***********************************************************************************************************************/
/***********************************************************************************************************************
* File Name    : rtpc.c
* Version      : 1.00
* Description  : This file is rBLE Application.
***********************************************************************************************************************/
/**********************************************************************************************************************
* History : DD.MM.YYYY Version Description
*         : 22.02.2019 1.00     First Release
***********************************************************************************************************************/

/**********************************************************************************************************************
* INCLUDE FILES
***********************************************************************************************************************/

#include    "arch.h"
#include    "rwble_config.h"

#include    "rble.h"
#include    "rble_app.h"

#include    <stdio.h>
#include    <string.h>
#include    "rble_api_rrtp.h"

#define RBLE_ATT_SVC_SMPL_CSTM      0xFFFF

typedef enum RTPC_STATUS_enum
{
    RTPC_STATUS_IDLE,
    RTPC_STATUS_DISCOVERING,
    RTPC_STATUS_CONNECTED
} RTPC_STATUS;

enum
{
    RTPC_DSC_NONE = 0x00,
    RTPC_DSC_NOTIFY_CFG,
    RTPC_DSC_DISCOVERY_DONE = 0xFF
};

enum
{
    RTPC_CHAR_NOTIFY = 0x00,
    RTPC_CHAR_OTHER = 0xFF
};

typedef struct RTPC_INFO
{
    uint16_t                conhdl;
    RTPC_STATUS             status;
    uint8_t                 cfg_dsc_idx;
    uint16_t                last_uuid_req;
    RBLE_RTPC_EVENT_HANDLER callback;
    RBLE_RRTS_CONTENT       rtps;
} RTPC_INFO_t;

/* internal function definitions */
static void gatt_cmp_evt_handler(RBLE_GATT_EVENT *event);
static void gatt_disc_svc_by_uuid_evt_handler(RBLE_GATT_EVENT *event);
static void gatt_disc_char_all_evt_handler(RBLE_GATT_EVENT *event);
static void gatt_disc_char_desc_evt_handler(RBLE_GATT_EVENT *event);
static void gatt_callback(RBLE_GATT_EVENT *event);

static void rtpc_disc_svc_send(void);
static void rtpc_disc_char_all_send(uint16_t shdl, uint16_t ehdl);
static void rtpc_disc_char_desc_send(uint16_t shdl);
static uint16_t rtpc_get_next_char_hdl(uint16_t shdl);
static uint8_t rtpc_get_char_index(uint8_t *uuid);

/* global variables definitions */
RTPC_INFO_t rtpc_info = {
    0,                      /* conhdl */
    RTPC_STATUS_IDLE,       /* status */
    RTPC_DSC_NONE,          /* cfg_dsc_idx */
    0,                      /* last_uuid_req */
    NULL,                   /* callback */
    {0}                     /* rtps */
};

/* 128bit UUID definitions */
const uint8_t rtpc_service_uuid[16]     = RBLE_SVC_RAPID_TRANSFER;
const uint8_t rtpc_notify_char_uuid[16] = RBLE_CHAR_RRTP_NOTIFY;

uint8_t g_hdl_set_flg = 0;

/**********************************************************************************************************************
* FUNCTION DEFINITIONS
***********************************************************************************************************************/

/***********************************************************************************************************************
* Function Name: RBLE_RRTP_Client_Enable
* Description  : This task is performed client enabling.
*
* Arguments    : (st_init_api_info_t *) init_api_info_t -
*                    The location of the pointer to put the device configuration data
*                (uint16_t) conhdl -
*                    The connection handler
*                (uint8_t) con_type -
*                    The type of connection
*                (RBLE_RRTS_CONTENT *) rtps -
*                    The pointer of parameter
*                (RBLE_RTPS_EVENT_HANDLER) call_back -
*                    The pointer of call back function
* Return Value : (RBLE_STATUS) result -
*                  - RBLE_OK  :00h - normal end
*                  - RBLE_PARAM_ERR :F3h - parameter error
*                  - RBLE_STATUS_ERROR :F2h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_RRTP_Client_Enable(uint16_t conhdl,
                                    uint8_t  con_type,
                                    RBLE_RRTS_CONTENT *rtps,
                                    RBLE_RTPC_EVENT_HANDLER call_back)
{
    RBLE_STATUS result = RBLE_OK;
    RBLE_RTPC_EVENT rrtp_event;
    
    if (RTPC_STATUS_IDLE == rtpc_info.status)
    {
        /* parameter check */
        if(call_back != NULL)
        {
            /* initialise GATT */
            result = RBLE_GATT_Enable(gatt_callback);
            if (RBLE_OK == result)
            {
                /* Reset the RTPC environment */
                memset(&rtpc_info, 0, sizeof(rtpc_info));
                
                /* regist callback function */
                rtpc_info.callback = call_back;
                
                /* set conhdl */
                rtpc_info.conhdl   = conhdl;

                /* config connection, start discovering */
                if (RBLE_RRTP_CON_CFG == con_type)
                {
                    /* start discovering SCS on peer */
                    rtpc_disc_svc_send();
                }
                /* normal connection, get saved att details */
                else
                {
                    rtpc_info.rtps = *rtps;

                    rtpc_info.status = RTPC_STATUS_CONNECTED;

                    /* enable complete callback */
                    rrtp_event.type = RBLE_RRTP_EVENT_CLIENT_ENABLE_COMP;
                    rrtp_event.param.client_enable.conhdl = conhdl;
                    rrtp_event.param.client_enable.status = RBLE_OK;
                    rrtp_event.param.client_enable.rtps    = rtpc_info.rtps;
                    
                    rtpc_info.callback(&rrtp_event);
                }
            }
        }
        else
        {
            /* parameter error */
            result = RBLE_PARAM_ERR;
        }
    }
    else
    {
        /* status error */
        result = RBLE_STATUS_ERROR;
    }
    return result;
}

/***********************************************************************************************************************
* Function Name: RBLE_RRTP_Client_Disable
* Description  : This task is performed client disabling.
*
* Arguments    : (uint16_t) conhdl -
*                    The connection handler
* Return Value : (RBLE_STATUS) result -
*                  - RBLE_OK  :00h - normal end
*                  - RBLE_PARAM_ERR :F3h - parameter error
*                  - RBLE_STATUS_ERROR :F2h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_RRTP_Client_Disable(uint16_t conhdl)
{
    RBLE_STATUS result = RBLE_OK;
    RBLE_RTPC_EVENT rrtp_event;
    
    if (RTPC_STATUS_CONNECTED == rtpc_info.status)
    {
        if(conhdl == rtpc_info.conhdl)
        {
            rtpc_info.status = RTPC_STATUS_IDLE;
            
            /* disable complete callback */
            rrtp_event.type = RBLE_RRTP_EVENT_CLIENT_DISABLE_COMP;
            rrtp_event.param.client_disable.conhdl = conhdl;
            rrtp_event.param.client_disable.status = RBLE_OK;
            
            rtpc_info.callback(&rrtp_event);
        }
        else
        {
            /* parameter error */
            result = RBLE_PARAM_ERR;
        }
    }
    else
    {
        /* status error */
        result = RBLE_STATUS_ERROR;
    }
    
    return result;
}

/***********************************************************************************************************************
* Function Name: RBLE_RRTP_Client_Read_Char
* Description  : This task is performed read characteristic value or descriptor specified by char_code.
*
* Arguments    : (uint16_t) conhdl -
*                    The connection handler
*                (uint8_t) char_code -
*                    The characteristic value or configuration descriptor
* Return Value : (RBLE_STATUS) result -
*                  - RBLE_OK  :00h - normal end
*                  - RBLE_PARAM_ERR :F3h - parameter error
*                  - RBLE_STATUS_ERROR :F2h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_RRTP_Client_Read_Char(uint16_t conhdl,
                                       uint8_t  char_code)
{
    RBLE_STATUS result = RBLE_OK;
    RBLE_GATT_READ_CHAR_REQ rd_char;
    uint16_t    search_hdl;
    
    if (RTPC_STATUS_CONNECTED == rtpc_info.status)
    {
        if(conhdl == rtpc_info.conhdl)
        {
            switch(char_code)
            {
                case RBLE_RRTP_RRTS_NTF_CFG:
                    search_hdl = rtpc_info.rtps.ntf_cfg_hdl;
                    rd_char.req_type    = RBLE_GATT_READ_DESC;
                    break;
                default:
                    result = RBLE_PARAM_ERR;
                    break;
            }
            
            if (RBLE_OK == result)
            {
                rd_char.conhdl      = conhdl;
                rd_char.uuid[0].value_size = 2;
                memcpy(&rd_char.uuid[0].value[0], &search_hdl, 2);
                
                result = RBLE_GATT_Read_Char_Request(&rd_char);
            }
        }
        else
        {
            /* parameter error */
            result = RBLE_PARAM_ERR;
        }
    }
    else
    {
        /* status error */
        result = RBLE_STATUS_ERROR;
    }
    
    return result;
}

/***********************************************************************************************************************
* Function Name: RBLE_RRTP_Client_Write_Char
* Description  : This task is performed write characteristic value or descriptor specified by char_code.
*
* Arguments    : (uint16_t) conhdl -
*                    The connection handler
*                (uint8_t) char_code -
*                    The characteristic value or configuration descriptor
*                (uint8_t *) write_value -
*                    The pointer of write value
* Return Value : (RBLE_STATUS) result -
*                  - RBLE_OK  :00h - normal end
*                  - RBLE_PARAM_ERR :F3h - parameter error
*                  - RBLE_STATUS_ERROR :F2h - status error
***********************************************************************************************************************/
RBLE_STATUS RBLE_RRTP_Client_Write_Char(uint16_t conhdl,
                                        uint8_t  char_code,
                                        uint8_t  *write_value)
{
    RBLE_STATUS result = RBLE_OK;
    RBLE_GATT_WRITE_CHAR_REQ wr_char;
    
    if (RTPC_STATUS_CONNECTED == rtpc_info.status)
    {
        if(conhdl == rtpc_info.conhdl)
        {
            switch(char_code)
            {
                case RBLE_RRTP_RRTS_NTF_CFG:
                    memcpy(&wr_char.charhdl, &rtpc_info.rtps.ntf_cfg_hdl, 2);
                    wr_char.req_type    = RBLE_GATT_WRITE_DESC;
                    wr_char.val_len     = 2;
                    memcpy(&wr_char.value[0], &write_value[0], 2);
                    break;
                default:
                    result = RBLE_PARAM_ERR;
                    break;
            }
            
            if (RBLE_OK == result)
            {
                wr_char.conhdl       = conhdl;
                wr_char.wr_offset    = 0;
                wr_char.auto_execute = FALSE;
                
                result = RBLE_GATT_Write_Char_Request(&wr_char);
            }
        }
        else
        {
            /* parameter error */
            result = RBLE_PARAM_ERR;
        }
    }
    else
    {
        /* status error */
        result = RBLE_STATUS_ERROR;
    }
    
    return result;
}

/***********************************************************************************************************************
* Function Name: gatt_cmp_evt_handler
* Description  : This task is complete event handler.
*
* Arguments    : (RBLE_GATT_EVENT *) event -
*                    The pointer of event
* Return Value : None
***********************************************************************************************************************/
static void gatt_cmp_evt_handler(RBLE_GATT_EVENT *event)
{
    if ((RBLE_ATT_ERR_ATTRIBUTE_NOT_FOUND == event->param.complete.att_code) ||
        (RBLE_ATT_ERR_NO_ERROR == event->param.complete.att_code) ||
        (RBLE_ATT_ERR_INVALID_HANDLE == event->param.complete.att_code))
    {
        switch(rtpc_info.last_uuid_req)
        {
            case RBLE_ATT_SVC_SMPL_CSTM:
                rtpc_disc_char_all_send(rtpc_info.rtps.shdl, rtpc_info.rtps.ehdl);
                break;
            case RBLE_DECL_CHARACTERISTIC:
                rtpc_disc_char_desc_send(rtpc_info.rtps.ntf_char_hdl);
                rtpc_info.cfg_dsc_idx = RTPC_DSC_NOTIFY_CFG;
                break;
            case RBLE_DESC_CLIENT_CHAR_CONF:
                {
                    RBLE_RTPC_EVENT rrtp_event;
                    
                    rtpc_info.cfg_dsc_idx = RTPC_DSC_DISCOVERY_DONE;
                    rtpc_info.status = RTPC_STATUS_CONNECTED;
                    
                    /* enable complete callback */
                    rrtp_event.type = RBLE_RRTP_EVENT_CLIENT_ENABLE_COMP;
                    rrtp_event.param.client_enable.conhdl = rtpc_info.conhdl;
                    rrtp_event.param.client_enable.status = RBLE_OK;
                    rrtp_event.param.client_enable.rtps    = rtpc_info.rtps;
                    
                    rtpc_info.callback(&rrtp_event);
                }
                break;
            default:
                __nop();
                break;
        }
    }
}

/***********************************************************************************************************************
* Function Name: gatt_disc_svc_by_uuid_evt_handler
* Description  : This task is svc by uuid event handler.
*
* Arguments    : (RBLE_GATT_EVENT *) event -
*                    The pointer of event
* Return Value : None
***********************************************************************************************************************/
static void gatt_disc_svc_by_uuid_evt_handler(RBLE_GATT_EVENT *event)
{
    if (RBLE_OK == event->param.disc_svc_by_uuid_cmp.att_code)
    {
        rtpc_info.rtps.shdl = event->param.disc_svc_by_uuid_cmp.list[0].start_hdl;
        rtpc_info.rtps.ehdl = event->param.disc_svc_by_uuid_cmp.list[0].end_hdl;
    }
}

/***********************************************************************************************************************
* Function Name: gatt_disc_char_all_evt_handler
* Description  : This task is characteristic by 128bit UUID complete event handler.
*
* Arguments    : (RBLE_GATT_EVENT *) event -
*                    The pointer of event
* Return Value : None
***********************************************************************************************************************/
static void gatt_disc_char_all_evt_handler(RBLE_GATT_EVENT *event)
{
    uint8_t char_idx;
    
    if (RBLE_OK == event->param.disc_char_all_128_cmp.att_code)
    {
        char_idx = rtpc_get_char_index(&event->param.disc_char_all_128_cmp.list.uuid[0]);
        
        switch(char_idx)
        {
            case RTPC_CHAR_NOTIFY:
                rtpc_info.rtps.ntf_char_hdl      = event->param.disc_char_all_128_cmp.list.attr_hdl;
                rtpc_info.rtps.ntf_prop          = event->param.disc_char_all_128_cmp.list.prop;
                rtpc_info.rtps.ntf_val_hdl       = event->param.disc_char_all_128_cmp.list.pointer_hdl;
                break;
            default:
                __nop();
                break;
        }
    }
}

/***********************************************************************************************************************
* Function Name: gatt_disc_char_desc_evt_handler
* Description  : This task is characteristic descriptor complete event handler.
*
* Arguments    : (RBLE_GATT_EVENT *) event -
*                    The pointer of event
* Return Value : None
***********************************************************************************************************************/
static void gatt_disc_char_desc_evt_handler(RBLE_GATT_EVENT *event)
{
    uint8_t i;
    
    for(i = 0; i < event->param.disc_char_desc_cmp.nb_entry; i++)
    {
        if (RBLE_DESC_CLIENT_CHAR_CONF == event->param.disc_char_desc_cmp.list[i].desc_hdl)
        {
            switch(rtpc_info.cfg_dsc_idx)
            {
                case RTPC_DSC_NOTIFY_CFG:
                    if(0 == g_hdl_set_flg){
                        rtpc_info.rtps.ntf_cfg_hdl = event->param.disc_char_desc_cmp.list[i].attr_hdl;
                        g_hdl_set_flg = 1;
                    }
                    break;
                default:
                    __nop();
                    break;
            }
        }
    }
}

/***********************************************************************************************************************
* Function Name: gatt_callback
* Description  : This task is entry of GATT callback
*
* Arguments    : (RBLE_GATT_EVENTi *) event -
*                    The pointer of structure of event
* Return Value : None
***********************************************************************************************************************/
static void gatt_callback(RBLE_GATT_EVENT *event)
{
    RBLE_RTPC_EVENT      rrtp_event;
    
    switch(event->type)
    {
        case RBLE_GATT_EVENT_COMPLETE:
            gatt_cmp_evt_handler(event);
            break;
        case RBLE_GATT_EVENT_DISC_SVC_BY_UUID_CMP:
            gatt_disc_svc_by_uuid_evt_handler(event);
            break;
        case RBLE_GATT_EVENT_DISC_CHAR_ALL_128_CMP:
            gatt_disc_char_all_evt_handler(event);
            break;
        case RBLE_GATT_EVENT_DISC_CHAR_DESC_CMP:
            gatt_disc_char_desc_evt_handler(event);
            break;
        case RBLE_GATT_EVENT_READ_CHAR_RESP:
            rrtp_event.type = RBLE_RRTP_EVENT_CLIENT_READ_CHAR_RESPONSE;
            rrtp_event.param.rd_char_resp.conhdl   = rtpc_info.conhdl;
            rrtp_event.param.rd_char_resp.att_code = event->param.read_char_resp.att_code;
            rrtp_event.param.rd_char_resp.data.len = event->param.read_char_resp.data.len;
            rrtp_event.param.rd_char_resp.data.each_len = event->param.read_char_resp.data.each_len;
            memcpy(&rrtp_event.param.rd_char_resp.data.data[0], &event->param.read_char_resp.data.data[0], event->param.read_char_resp.data.len);
            
            rtpc_info.callback(&rrtp_event);
            break;
        case RBLE_GATT_EVENT_WRITE_CHAR_RESP:
            rrtp_event.type = RBLE_RRTP_EVENT_CLIENT_WRITE_CHAR_RESPONSE;
            rrtp_event.param.wr_char_resp.conhdl   = rtpc_info.conhdl;
            rrtp_event.param.wr_char_resp.att_code = event->param.write_char_resp.att_code;
            
            rtpc_info.callback(&rrtp_event);
            break;
        case RBLE_GATT_EVENT_HANDLE_VALUE_NOTIF:
            rrtp_event.type = RBLE_RRTP_EVENT_CLIENT_NOTIFY;
            rrtp_event.param.notify.conhdl   = rtpc_info.conhdl;
            rrtp_event.param.notify.data_len = event->param.handle_value_notif.size;
            memcpy(&rrtp_event.param.notify.data[0], &event->param.handle_value_notif.value[0], event->param.handle_value_notif.size);
            
            rtpc_info.callback(&rrtp_event);
            break;
        case RBLE_GATT_EVENT_HANDLE_VALUE_IND:
        case RBLE_GATT_EVENT_RESP_TIMEOUT:
        case RBLE_GATT_EVENT_COMMAND_DISALLOWED_IND:
        case RBLE_GATT_EVENT_DISC_CHAR_DESC_128_CMP:
        default:
            rrtp_event.type = RBLE_RRTP_EVENT_CLIENT_INDICATE;
            break;
    }
}

/***********************************************************************************************************************
* Function Name: rtpc_disc_svc_send
* Description  : This function is send svc
*
* Arguments    : None
* Return Value : None
***********************************************************************************************************************/
static void rtpc_disc_svc_send(void)
{
    RBLE_GATT_DISC_SVC_REQ disc_svc;
    
    disc_svc.req_type   = RBLE_GATT_DISC_BY_UUID_SVC;
    disc_svc.conhdl     = rtpc_info.conhdl;
    disc_svc.start_hdl  = 0x0001u;
    disc_svc.end_hdl    = 0xFFFFu;
    disc_svc.desired_svc.value_size = 16;
    memcpy(&disc_svc.desired_svc.value[0], &rtpc_service_uuid, sizeof(rtpc_service_uuid));
    
    (void)RBLE_GATT_Discovery_Service_Request(&disc_svc);
    
    rtpc_info.last_uuid_req = RBLE_ATT_SVC_SMPL_CSTM;
}

/***********************************************************************************************************************
* Function Name: rtpc_disc_char_all_send
* Description  : This function is all send descriptor
*
* Arguments    : (uint16_t) shdl -
*                    The start handle
*                (uint16_t) ehdl -
*                    The end handle
* Return Value : None
***********************************************************************************************************************/
static void rtpc_disc_char_all_send(uint16_t shdl, uint16_t ehdl)
{
    RBLE_GATT_DISC_CHAR_REQ disc_char = {0};
    
    disc_char.req_type  = RBLE_GATT_DISC_ALL_CHAR;
    disc_char.conhdl    = rtpc_info.conhdl;
    disc_char.start_hdl = shdl;
    disc_char.end_hdl   = ehdl;
    
    (void)RBLE_GATT_Discovery_Char_Request(&disc_char);
    
    rtpc_info.last_uuid_req = RBLE_DECL_CHARACTERISTIC;
}

/***********************************************************************************************************************
* Function Name: rtpc_disc_char_desc_send
* Description  : This function is send descriptor
*
* Arguments    : (uint16_t) shdl -
*                    The start handle
* Return Value : None
***********************************************************************************************************************/
static void rtpc_disc_char_desc_send(uint16_t shdl)
{
    RBLE_GATT_DISC_CHAR_DESC_REQ disc_char_desc = {0};
    
    disc_char_desc.conhdl    = rtpc_info.conhdl;
    disc_char_desc.start_hdl = shdl;
    disc_char_desc.end_hdl   = rtpc_get_next_char_hdl(shdl);
    
    (void)RBLE_GATT_Discovery_Char_Descriptor_Request(&disc_char_desc);
    
    rtpc_info.last_uuid_req = RBLE_DESC_CLIENT_CHAR_CONF;
}

/***********************************************************************************************************************
* Function Name: rtpc_get_next_char_hdl
* Description  : This function is get next handle
*
* Arguments    : (uint16_t) shdl -
*                    The start handle
* Return Value : None
***********************************************************************************************************************/
static uint16_t rtpc_get_next_char_hdl(uint16_t shdl)
{
    uint16_t hdlarr[5];
    uint16_t ret_hdl = rtpc_info.rtps.ehdl;
    uint8_t  i;

    hdlarr[0] = rtpc_info.rtps.ntf_char_hdl;
    
    //go through list to sort
    for(i=0; i<5; i++)
    {
        if( (shdl < hdlarr[i]) && (hdlarr[i] < ret_hdl) )
        {
            ret_hdl = hdlarr[i];
        }
    }
    return ret_hdl;
}

/***********************************************************************************************************************
* Function Name: rtpc_get_char_index
* Description  : This function is get index
*
* Arguments    : (uint8_t *) uuid -
*                    The pointer of uuid
* Return Value : None
***********************************************************************************************************************/
static uint8_t rtpc_get_char_index(uint8_t *uuid)
{
    uint8_t idx = RTPC_CHAR_OTHER;
    
    if(0 == memcmp(uuid, &rtpc_notify_char_uuid[0], sizeof(rtpc_notify_char_uuid)))
    {
        idx = RTPC_CHAR_NOTIFY;
    }
    else
    {
        idx = RTPC_CHAR_OTHER;
    }
    
    return idx;
}

